"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_observable_1 = __importDefault(require("axios-observable"));
const semver_1 = __importDefault(require("semver"));
const https_1 = __importDefault(require("https"));
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const util_1 = require("./util");
const MAX_CONTAINER_TAGS = 999999999;
function pollImageForUpdates(allowInsecureHttps, auth, pollableImage, pollDueToSignal) {
    var _a;
    const tagUrl = `https://${pollableImage.repoUrl}/v2/${pollableImage.image}/tags/list`;
    const axiosOptions = {
        params: { n: MAX_CONTAINER_TAGS },
        auth: (_a = auth === null || auth === void 0 ? void 0 : auth.get(pollableImage.repoUrl)) !== null && _a !== void 0 ? _a : undefined,
    };
    if (allowInsecureHttps) {
        axiosOptions.httpsAgent = new https_1.default.Agent({ rejectUnauthorized: false });
    }
    if (pollDueToSignal) {
        console.log("Polling for update due to signal: %O");
    }
    else {
        util_1.debugLog("Polling for update on schedule: %O", pollableImage);
    }
    return axios_observable_1.default.get(tagUrl, axiosOptions).pipe(operators_1.map(result => {
        var _a, _b;
        const tags = (_b = (_a = result.data) === null || _a === void 0 ? void 0 : _a.tags) !== null && _b !== void 0 ? _b : [];
        const spec = `^${pollableImage.tag}`;
        const nextTag = semver_1.default.maxSatisfying(tags, spec);
        if (!nextTag || nextTag === pollableImage.tag) {
            return undefined;
        }
        return {
            ...pollableImage,
            tag: nextTag,
        };
    }), operators_1.filter(util_1.isDefined), operators_1.catchError(error => {
        console.warn("Error polling container registry: %O", (error === null || error === void 0 ? void 0 : error.message) || error);
        return rxjs_1.empty();
    }));
}
function pollImagesForUpdate(initialPollableImages, pollFrequency, allowInsecureHttps, auth) {
    return rxjs_1.merge(...initialPollableImages.map(initialPollableImage => util_1.interruptableInterval(pollFrequency * 1000).pipe(util_1.switchScan(pollImageForUpdates.bind(null, allowInsecureHttps, auth), initialPollableImage)))).pipe(operators_1.scan((pollableImages, changedImage) => {
        const newImages = pollableImages.filter(pollableImage => pollableImage.repoUrl !== changedImage.repoUrl ||
            pollableImage.image !== changedImage.image);
        newImages.push(changedImage);
        return newImages;
    }, initialPollableImages));
}
exports.default = pollImagesForUpdate;
//# sourceMappingURL=pollImagesForUpdate.js.map