"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
class Process {
    constructor(proc, commandAndArgs, showStderr) {
        this.proc = proc;
        this.shutdownPromise = new Promise((resolve, reject) => {
            let stderr = "";
            proc.stderr.on("data", (stderrLine) => {
                if (showStderr) {
                    console.warn(stderrLine.toString().trimRight());
                }
                stderr += stderrLine;
            });
            proc.on("close", (exitCode) => {
                if (!exitCode) {
                    resolve();
                }
                else {
                    const prefix = `Error running "${commandAndArgs.join(" ")}": `;
                    reject(new Error(prefix + (stderr || "Unknown error")));
                }
            });
        });
    }
    shutdown() {
        this.proc.kill("SIGTERM");
        return this.wait();
    }
    wait() {
        return this.shutdownPromise;
    }
}
exports.Process = Process;
exports.spawnProcess = (commandAndArgs, options = {}) => {
    const spawnOptions = {};
    spawnOptions.stdio = [
        "pipe",
        options.showStdout ? "inherit" : "ignore",
        "pipe",
    ];
    const proc = child_process_1.spawn(commandAndArgs[0], commandAndArgs.slice(1), spawnOptions);
    return new Process(proc, commandAndArgs, options.showStderr);
};
exports.runCommand = (commandAndArgs, options = {}) => exports.spawnProcess(commandAndArgs, options).wait();
exports.execCommand = (commandAndArgs) => new Promise((resolve, reject) => {
    const proc = child_process_1.spawn(commandAndArgs[0], commandAndArgs.slice(1), {
        stdio: ["ignore", "pipe", "pipe"],
    });
    let stdout = "";
    proc.stdout.on("data", (stdoutLine) => {
        stdout += stdoutLine;
    });
    let stderr = "";
    proc.stderr.on("data", (stderrLine) => {
        stderr += stderrLine;
    });
    proc.on("close", (exitCode) => {
        if (!exitCode) {
            resolve(stdout);
        }
        else {
            const prefix = `Error running "${commandAndArgs.join(" ")}": `;
            reject(new Error(prefix + (stderr || "Unknown error")));
        }
    });
});
//# sourceMappingURL=processes.js.map